#-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$
# risk_to_road.py
#
# By Michael Bunn, Oregon State University
# Created on 02/09/2018
#
#-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$-$

# Import modules, initiate arcpy
import arcpy
from arcpy import env
from arcpy.sa import *
arcpy.env.overwriteOutput = True
arcpy.CheckOutExtension("Spatial")

# Define functions
# Define new addField function
def addField(Features,Fieldname,Type):
	check = 0
	lst = arcpy.ListFields(Features)
	for f in lst:
		if f.name == Fieldname:
			check = 1
		else:
			continue
	if check == 0:
		arcpy.AddField_management(Features,Fieldname,Type)

# Get user inputs
input_raster = arcpy.GetParameterAsText(0)
input_roads = arcpy.GetParameterAsText(1)
output_roads = arcpy.GetParameterAsText(2)

# Get optional user inputs
num_zones = long(arcpy.GetParameter(3))
slice_type = arcpy.GetParameterAsText(4)
search_dist = arcpy.GetParameter(5)
search_for = arcpy.GetParameterAsText(6)

# Check whether or not to perform Focal Stats
if search_dist != '':
	# Perform Focal Statistics
	tempFocal = "tempFocal"
	outFocalStat = FocalStatistics(input_raster,NbrCircle(search_dist,"MAP"),search_for)
	outFocalStat.save(tempFocal)
else:
	tempFocal = input_raster

# Perform Slice
tempSlice = "tempSlice"
outSlice = Slice(tempFocal,num_zones,slice_type)
outSlice.save(tempSlice)

# Perform Zonal Statistics as Table to get zone values
outZSat = ZonalStatisticsAsTable(tempSlice,"VALUE",tempFocal,"zonalstattblout","NODATA","ALL")

if search_dist != '':
	arcpy.Delete_management(tempFocal)

# Convert raster to polygons
tempPoly = "tempPoly"
arcpy.RasterToPolygon_conversion(tempSlice, tempPoly, "SIMPLIFY")
arcpy.Delete_management(tempSlice)

# Create polygon layer and join to ZS table
arcpy.MakeFeatureLayer_management(tempPoly,"polyLyr")
arcpy.AddJoin_management("polyLyr","gridcode","zonalstattblout","Value")

# Intersect roads with polygon
tempRoads = "tempRoads"
arcpy.Intersect_analysis([input_roads,"polyLyr"],tempRoads)

arcpy.Delete_management("polyLyr")
arcpy.Delete_management(tempPoly)
arcpy.Delete_management("zonalstattblout")

arcpy.Dissolve_management(tempRoads,output_roads,["zonalstattblout_Value","zonalstattblout_MEAN","zonalstattblout_MAX","zonalstattblout_MIN"])

arcpy.CheckInExtension("Spatial")

# Add output layer to the map

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]

# Create layer file and add to map
lyrRoads = arcpy.mapping.Layer(output_roads)
arcpy.mapping.AddLayer(df,lyrRoads)

arcpy.CheckInExtension("spatial")